import sys
import pygame
from pygame.locals import *
from random import seed

import Bill
import Bucket
import Computer
import Horde
import Network
import OS
import Spark
import UI
from util import fatal

SCREENSIZE = 400

STATE_PLAYING = 1
STATE_BETWEEN = 2
STATE_END = 3
STATE_WAITING = 4

SCORE_ENDLEVEL = -1
SCORE_BILLPOINTS = 5

class Game:
	def __init__(self):
		self.screensize = SCREENSIZE
		self.state = STATE_WAITING
		self.level = 1

	def setup_level(self, newlevel):
		self.level = newlevel
		self.horde.setup(self)
		self.network.setup(self)

		self.grabbed = None
		UI.set_cursor(self.defaultcursor)
		self.iteration = 0
		self.efficiency = 0

	def start(self, newlevel):
		self.state = STATE_PLAYING
		self.score = 0
		self.setup_level(newlevel)

	def update_info(self):
		on_screen = self.horde.counters[Horde.COUNTER_ON]
		off_screen = self.horde.counters[Horde.COUNTER_OFF]
		base = self.network.counters[Network.COUNTER_BASE]
		off = self.network.counters[Network.COUNTER_OFF]
		win = self.network.counters[Network.COUNTER_WIN]
		units = len(self.network.computers)
		str = "Bill:%d/%d  System:%d/%d/%d  Level:%d  Score:%d" % (on_screen, off_screen, base, off, win, self.level, self.score)
		self.draw_str(str, 5, self.screensize - 15)
		self.efficiency += ((100 * base - 10 * win) / units)

	def warp_to_level(self):
		if self.state == STATE_PLAYING:
			if lev <= self.level:
				return
			self.setup_level(lev)
		else:
			if lev <= 0:
				return
			self.start(lev)

	def button_press(self, x, y):
		if self.state != STATE_PLAYING:
			return
		UI.set_cursor(self.downcursor)
	
		if self.bucket.clicked(x, y):
			self.bucket.grab()
			return
	
		self.grabbed = self.horde.clicked_stray(x, y)
		if self.grabbed != None:
			OS.set_cursor(self.grabbed.cargo)
			return
	
		counter = self.horde.process_click(x, y)
		self.score += (counter * counter * SCORE_BILLPOINTS)

	def button_release(self, x, y):
		UI.set_cursor(self.defaultcursor)
	
		if self.state != STATE_PLAYING:
			return
	
		if self.grabbed == None:
			self.bucket.release(x, y, self.network.cables)
			return
	
		for computer in self.network.computers:
			if computer.on(x, y) and computer.compatible(self.grabbed.cargo) and (computer.os == OS.WINGDOWS or computer.os == OS.OFF):
				self.network.counters[Network.COUNTER_BASE] += 1
				if computer.os == OS.WINGDOWS:
					counter = Network.COUNTER_WIN
				else:
					counter = Network.COUNTER_OFF
				self.network.counters[counter] += -1
				computer.os = self.grabbed.cargo
				self.horde.remove_bill(self.grabbed, self.network)
				self.grabbed = None
				return
		self.horde.add_bill(self.grabbed)
		self.grabbed = None

	def draw_logo(self):
		self.screen.fill((255, 255, 255))
		self.screen.blit(self.logo, ((self.screensize - self.logo.get_width()) / 2, (self.screensize - self.logo.get_height()) / 2))
		UI.refresh()

	def update(self):
		if self.state == STATE_PLAYING:
			self.screen.fill((255, 255, 255))
			self.bucket.draw(self.screen)
			self.network.update()
			self.network.draw(self.screen)
			self.horde.update(self)
			self.horde.draw(self.screen)
			self.update_info()
			if self.horde.counters[Horde.COUNTER_ON] + self.horde.counters[Horde.COUNTER_OFF] == 0:
				self.score += (self.level * self.efficiency / self.iteration)
				self.state = STATE_BETWEEN
			if self.network.counters[Network.COUNTER_BASE] + self.network.counters[Network.COUNTER_OFF] <= 1:
				self.state = STATE_END
	
		elif self.state == STATE_END:
			UI.set_cursor(self.defaultcursor)
			self.screen.fill((255, 255, 255))
			self.network.toasters()
			self.network.draw(self.screen)
			UI.refresh()
	
		elif self.state == STATE_BETWEEN:
			UI.set_cursor(self.defaultcursor)
			self.state = STATE_PLAYING
			self.level += 1
			self.setup_level(self.level)

		UI.refresh()
		self.iteration += 1

	def scale(self, dimensions):
		scale = float(self.screensize) / float(SCREENSIZE)
		d = 1.0
		for None in range(dimensions):
			d *= scale
		return d

	def draw_str(self, str, x, y):
		surface = self.font.render(str, 1, (0, 0, 0))
		self.screen.blit(surface, (x, y))

	def main_loop(self):
		clock = pygame.time.Clock()
		while 1:
			for event in pygame.event.get():
				if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE):
					return
				elif event.type is MOUSEBUTTONDOWN:
					pos = pygame.mouse.get_pos()
					self.button_press(pos[0], pos[1])
				elif event.type is MOUSEBUTTONUP:
					pos = pygame.mouse.get_pos()
					self.button_release(pos[0], pos[1])
			self.update()
			pygame.event.pump()
			pygame.time.wait(200)

#
# Note - don't use getopt, since it might reorder the args or do something
# that the UI-specific argument parser doesn't like.
#
def parse_args(argv, game):
	for i in range(len(argv)):
		if argv[i] in ("-l", "-L"):
			try:
				i += 1
				game.level = int(argv[i])
			except IndexError:
				fatal("-l takes an argument")
			except ValueError:
				fatal("invalid level '%s'" % game.level)
		elif argv[i] == "--size":
			try:
				i += 1
				game.screensize = int(argv[i])
			except IndexError:
				fatal("--size takes an argument")
			except ValueError:
				fatal("invalid screensize '%s'" % game.screensize)
			if game.screensize < SCREENSIZE:
				fatal("screensize must be larger than '%d'" % SCREENSIZE)
		elif argv[i] in ("-v", "--version"):
			print "PyBill version 2.1"
			sys.exit(0)
		elif argv[i] in ("-h", "--help"):
			print """\
PyBill version 2.1RC1
Options:
print "-l <n>\tStart at level n.
--size <size>\t\tUse a larger playing area.
-v\t\tPrint version number and exit.
-h\t\tPrint help and exit.
All standard toolkit options are also supported.
"""
			sys.exit(0)

def main():
	seed()
	game = Game()
	parse_args(sys.argv, game)
	pygame.init()
	game.screen = pygame.display.set_mode((game.screensize, game.screensize))
	game.font = pygame.font.Font(None, 20)
	game.logo = UI.load_picture("logo")
	icon = UI.load_picture("logo")
	game.draw_logo()
	UI.refresh()
	UI.set_icon(icon)
	
	game.defaultcursor = UI.load_cursor("hand_up", UI.CURSOR_SEP_MASK)
	game.downcursor = UI.load_cursor("hand_down", UI.CURSOR_SEP_MASK)
	UI.set_cursor(game.defaultcursor)

	Bill.load_pix()
	OS.load_pix()
	Computer.load_pix()
	Bucket.load_pix()
	Spark.load_pix()

	game.bucket = Bucket.Bucket()
	game.horde = Horde.Horde()
	game.network = Network.Network()

	game.start(game.level)
	game.main_loop()
	sys.exit(0)

if __name__ == '__main__':
	main()
